/**
 * @jest-environment jsdom
 */

// Copyright (c) 2023 Tudor Stanciu

import axios from "axios";
import { TuitioClient, fetch } from "../client";

jest.mock("axios");

test("Tuitio client: successfully account login", async () => {
  (axios.request as jest.Mock).mockResolvedValue({
    data: {
      result: {
        token: "token-mock",
        expiresIn: 600000
      },
      error: null
    }
  });

  const client = new TuitioClient("https://test.com/api");
  const loginResult = await client.login("user", "pass");

  expect(loginResult.result.token).toBe("token-mock");
  expect(loginResult.result.expiresIn).toBe(600000);
  expect(loginResult.error).toBeNull();

  const storage = fetch();
  expect(storage).toBeDefined();
  expect(storage?.token).toBe("token-mock");
  expect(storage?.userName).toBe("user");
});

test("Tuitio client failed authentication", async () => {
  (axios.request as jest.Mock).mockRejectedValueOnce({
    response: {
      status: 500,
      data: { title: "Internal server error" }
    }
  });

  const client = new TuitioClient("https://test.com/api");

  try {
    await client.login("user", "pass");
  } catch (error: any) {
    expect(error.title).toBe("Internal server error");
    expect(error.message).toBe("Internal server error");
  }
});

test("Tuitio client failed authentication with unhandled exception", async () => {
  (axios.request as jest.Mock).mockRejectedValueOnce({
    response: {
      status: 500,
      error: { message: "Internal server error" }
    }
  });

  const client = new TuitioClient("https://test.com/api");

  try {
    await client.login("user", "pass");
  } catch (error: any) {
    expect(error.response.status).toBe(500);
    expect(error.response.error.message).toBe("Internal server error");
  }
});
